import os
import json
from django.core.management.base import BaseCommand
from textmanager.models import Language


class Command(BaseCommand):
    help = "Load language list to database"

    def handle(self, *args, **options):
        with open(f'{os.path.dirname(os.path.realpath(__file__))}/languages.json', 'r') as f:
            languages = json.load(f)
            for language in languages:
                try:
                    Language(**language).save()
                except Exception as e:
                    self.stdout.write(
                        self.style.ERROR(f'Error with load "{language}", exception: {e}')
                    )
        
        self.stdout.write(
            self.style.SUCCESS(f'{len(languages)} languages loaded')
        )



from typing import Union, List, Tuple

from django.db import models
from django.db.models import Q
from django.contrib.auth.models import Group

import jinja2


class Language(models.Model):
    alpha2 = models.CharField(max_length=2, unique=True)
    alpha3_b = models.CharField(max_length=3, unique=True)
    english_name = models.CharField(max_length=30, unique=True)
    language_name = models.CharField(max_length=30, unique=True)
    flag = models.CharField(max_length=10)

    def __str__(self) -> int:
        return f"{self.english_name}{self.flag}"
    
    
class Category(models.Model):
    unique_id = models.CharField(max_length=255, unique=True)
    title = models.CharField(max_length=255, unique=True)

    groups = models.ManyToManyField(to=Group, blank=True)

    def __str__(self) -> str:
        return f"{self.title}"

    class Meta:
        verbose_name_plural = 'categories'


class Text(models.Model):
    unique_id = models.CharField(max_length=255, unique=True, null=True, blank=True)
    category = models.ForeignKey(to=Category, on_delete=models.CASCADE, null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    
    def render(self, language: Union[str, Language, None], params: dict, render_with_jinja=True) -> Union[str, List[Tuple[str, str]]]:
        if isinstance(language, str):
            language = Language.objects.filter(Q(alpha2=language) | Q(alpha3_b=language)).first()
        if not isinstance(language, Language):
            language = None
        
        if language is not None:
            language_text = LanguageText.objects.filter(language=language, text=self).first()
            if language_text is not None:
                return language_text.render(params=params, render_with_jinja=render_with_jinja)
            return None
        
        texts = list()
        for language_text in LanguageText.objects.filter(text=self).all():
            texts.append((
                language_text.language, language_text.render(params=params, render_with_jinja=render_with_jinja)
            ))
        
        return texts
    
    def __str__(self) -> str:
        return f"Text({self.unique_id})"


class LanguageText(models.Model):
    language = models.ForeignKey(to=Language, on_delete=models.CASCADE)
    text = models.ForeignKey(to=Text, on_delete=models.CASCADE)

    value = models.TextField()

    def render(self, params: dict, render_with_jinja=True):
        if render_with_jinja:
            return jinja2.Template(self.value).render(**params)
        return self.value

    class Meta:
        unique_together = ('language', 'text',)


import json
from django.http import HttpRequest, JsonResponse

from rest_framework.views import APIView
from rest_framework import permissions

from textmanager.serializers import TextParamsSerializer, LanguageSerializer
from textmanager.models import Language, Text
from textmanager import queryset_manager, responses


class TextView(APIView):
    permission_classes = (permissions.IsAuthenticated,)

    def post(self, request: HttpRequest):
        data = TextParamsSerializer(data=json.loads(request.body))
        if not data.is_valid():
            return JsonResponse(data.errors)
        
        language = data.validated_data['language']
        unique_id = data.validated_data['unique_id']
        render_with_jinja = data.validated_data['render_with_jinja']
        params = data.validated_data['params']

        if language is not None:
            qs = queryset_manager.available_languages_queryset_filter(Language.objects)
            language = queryset_manager._filter_languages(qs, [data.validated_data['language']]).first()
            if language is None:
                return responses.LANGUAGE_NOT_FOUND
        
        text = queryset_manager.available_for_user_text_queryset_filter(
            Text.objects, request.user
        ).filter(unique_id=unique_id).first()
        
        if text is None:
            return responses.TEXT_NOT_FOUND
        
        rendered_text = text.render(language=language, params=params, render_with_jinja=render_with_jinja)

        if rendered_text is None:
            return responses.TEXT_NOT_FOUND
        
        if not isinstance(rendered_text, list):
            return JsonResponse({"text": rendered_text})
        else:
            texts = [{
                "language": LanguageSerializer().to_representation(language),
                "text": text
            } for language, text in rendered_text]
            return JsonResponse({"texts": texts})


from django.forms.models import BaseInlineFormSet
from textmanager.models import Language
from textmanager import queryset_manager


def language_text_inlile_formset_factory(text):
    class LanguageTextInlineFormSet(BaseInlineFormSet):
        def __init__(self, *args, **kwargs):
            qs = queryset_manager.extra_languages_queryset_filter(Language.objects, text)
            if qs is not None:
                kwargs['initial'] = [
                    {'language': i}
                    for i in qs
                ]
            super(LanguageTextInlineFormSet, self).__init__(*args, **kwargs)
    
    return LanguageTextInlineFormSet


from typing import List, Union

from django.conf import settings
from django.contrib.auth.models import User
from django.db.models.query import QuerySet
from django.db.models import Q

from textmanager.models import Language, Text, LanguageText


def _filter_languages(qs: QuerySet[Language], ids: Union[None, List[str]]) -> QuerySet[Language]:
    if ids is None:
        return qs
    
    return qs.filter(Q(alpha2__in=ids) | Q(alpha3_b__in=ids))
    
def available_languages_queryset_filter(qs: QuerySet[Language]) -> QuerySet[Language]:
    return _filter_languages(qs, settings.TEXT_MANAGER_AVAILABLE_LANGUAGES)

def extra_languages_queryset_filter(qs: QuerySet[Language], text: Text) -> QuerySet[Language]:
    excluded_languages = []
    if text is not None:
        excluded_languages = [x.language for x in LanguageText.objects.filter(text=text).all()]
    return _filter_languages(qs, settings.TEXT_MANAGER_EXTRA_LANGUAGES).exclude(id__in=[x.id for x in excluded_languages])

def available_for_user_text_queryset_filter(qs: QuerySet[Text], user: User) -> QuerySet[Text]:
    if user.is_superuser:
        return qs

    return qs.filter(category__groups__in=user.groups.all())



from typing import Any
from django.contrib import admin
from django.db.models.query import QuerySet
from django.http import HttpRequest
from textmanager.models import Language, Text, Category, LanguageText
from textmanager import queryset_manager
from textmanager import formsets


class LanguageAdmin(admin.ModelAdmin):
    list_display = ('alpha2', 'alpha3_b', 'english_name', 'language_name', 'flag')
    search_fields = list_display

    def get_queryset(self, request: HttpRequest) -> QuerySet[Any]:
        return queryset_manager.available_languages_queryset_filter(super().get_queryset(request))


class CategoryAdmin(admin.ModelAdmin):
    list_display = ('unique_id', 'title')
    search_fields = list_display
    filter_horizontal = ('groups', )


class LanguageTextInline(admin.TabularInline):
    model = LanguageText
    autocomplete_fields = ('language',)

    def get_formset(self, request: HttpRequest, obj=None, **kwargs: Any) -> Any:
        kwargs['formset'] = formsets.language_text_inlile_formset_factory(obj)
        return super().get_formset(request, obj, **kwargs)
    
    def get_extra(self, request: HttpRequest, obj=None, **kwargs: Any) -> int:
        return queryset_manager.extra_languages_queryset_filter(Language.objects, obj).count()


class TextAdmin(admin.ModelAdmin):
    list_display = ('unique_id', 'category', 'description')
    list_filter = ('category',)
    autocomplete_fields = ('category',)
    inlines = [LanguageTextInline,]

    def get_queryset(self, request: HttpRequest) -> QuerySet[Any]:
        qs = super().get_queryset(request)
        return queryset_manager.available_for_user_text_queryset_filter(qs, request.user)


admin.site.register(Language, LanguageAdmin)
admin.site.register(Category, CategoryAdmin)
admin.site.register(Text, TextAdmin)


from django.conf import settings

def configure_default_setting(default_settings: dict):
    for k, v in default_settings.items():
        try:
            getattr(settings, k)
        except:
            setattr(settings, k, v)


configure_default_setting({
    "TEXT_MANAGER_EXTRA_LANGUAGES": [], # In text admin extra inline items with languages
    "TEXT_MANAGER_AVAILABLE_LANGUAGES": None, # For filtering in queryset in languages admin
    "TEXT_MANAGER_DEFAULT_API_LANGUAGE": None, # Default language for api, fr. en to render english text if 'language' row in empty
})

if settings.TEXT_MANAGER_AVAILABLE_LANGUAGES is not None:
    settings.TEXT_MANAGER_EXTRA_LANGUAGES = list(set(settings.TEXT_MANAGER_EXTRA_LANGUAGES) & set(settings.TEXT_MANAGER_EXTRA_LANGUAGES))



# Generated by Django 4.1.3 on 2024-05-17 23:28

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="Category",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("unique_id", models.CharField(max_length=255, unique=True)),
                ("title", models.CharField(max_length=255, unique=True)),
                ("groups", models.ManyToManyField(blank=True, to="auth.group")),
            ],
            options={
                "verbose_name_plural": "categories",
            },
        ),
        migrations.CreateModel(
            name="Language",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("alpha2", models.CharField(max_length=2, unique=True)),
                ("alpha3_b", models.CharField(max_length=3, unique=True)),
                ("english_name", models.CharField(max_length=30, unique=True)),
                ("language_name", models.CharField(max_length=30, unique=True)),
                ("flag", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="Text",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "unique_id",
                    models.CharField(
                        blank=True, max_length=255, null=True, unique=True
                    ),
                ),
                ("description", models.TextField(blank=True, null=True)),
                (
                    "category",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="textmanager.category",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="LanguageText",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value", models.TextField()),
                (
                    "language",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="textmanager.language",
                    ),
                ),
                (
                    "text",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="textmanager.text",
                    ),
                ),
            ],
            options={
                "unique_together": {("language", "text")},
            },
        ),
    ]



from django.conf import settings
from rest_framework import serializers



class TextParamsSerializer(serializers.Serializer):
    unique_id = serializers.CharField()
    language = serializers.CharField(default=settings.TEXT_MANAGER_DEFAULT_API_LANGUAGE, allow_null=True)
    render_with_jinja = serializers.BooleanField(default=True)
    params = serializers.JSONField(default={})


class LanguageSerializer(serializers.Serializer):
    alpha2 = serializers.CharField()
    alpha3_b = serializers.CharField()
    english_name = serializers.CharField()
    language_name = serializers.CharField()
    flag = serializers.CharField()